package org.jconfig.items;

import java.io.*;

/**
 *  <p>package:    org.jconfig.items</p>
 *  <p>file:       ItemToXML.java</p>
 * $Id: ItemToXML.java 274 2004-04-19 08:12:45Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *	Item to XML converter
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class ItemToXML {
    
    private int             level;
    private StringWriter    xml;
    
    public ItemToXML() {
        level = 0;
        xml = new StringWriter();
        xml.write ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    }
    
    public void PrintIndent () {
        xml.write('\n');
        for (int i=0; i < level; i++)
                xml.write('\t');
    }
    
    private String Encode (String content) {
        try {
            return java.net.URLEncoder.encode(content, "UTF-8");
        } catch (Exception e) { 
            System.out.println(content);
        }
        return content;
    }
    
    public void StartTag (TreeItem item, boolean savearray) {
        int i;
        PrintIndent();
        if (item.GetConfig() != null)
            xml.write("<" + Encode(item.GetConfig().GetName()));
        else
            xml.write("<" + Encode(item.GetName()));
        for (i = 0; i < item.attr.size(); i++) {
            ItemAttr att = (ItemAttr)item.attr.get(i);
            if (!savearray && att.getName().compareTo("arraysize") == 0)
                continue;
            xml.write (" "+ Encode(att.getName()) + "=\"" + Encode(att.getValue()) +"\"");
        }
        xml.write (">" + Encode(item.GetContent()));
        level++;
    }
    
    public void EndTag (TreeItem item, boolean mustindent) {
        level--;
        if (mustindent)
            PrintIndent();
        if (item.GetConfig() != null)
            xml.write ("</" + Encode(item.GetConfig().GetName()) + ">");
        else
            xml.write ("</" + Encode(item.GetName()) + ">");
    }
    
    public String GetString() throws UnsupportedEncodingException {
        return xml.toString();
    }
}
