package org.jvisu.base;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values in a bar graph
 * <p>
 * $Id: JVBarMinMax.java 318 2007-08-21 11:23:19Z mol $
 * </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * <li> 1.1 Remove the JProgressBar stuff and make it based on JVScales
 * </ul>
 * <p>
 * </p>
 *  JVBarMinMax is used to display numerical values as a bar with
 *  embedded min / max history bars
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.1
 */
public class JVBarMinMax extends JVBar {
    
    /** The stored minimum value */
    protected double            minHistory = 0;
    /** The stored maximum value */
    protected double            maxHistory = 0;
    /** The color for min */
    protected Color             minColor = Color.ORANGE;
    /** The color for max */
    protected Color             maxColor = Color.RED;
    
    /** Create a new instance of JVBar
     */
    public JVBarMinMax() {
        /* Set super first ! */
        super();
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, 3));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, 4));
    }

    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
    }
    
    /** Paint this component
     *  @param g the graphics context
     */
    public void paintComponent (Graphics g) {

        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        
        /* Recalc the scale and the bar bounds */
        calcScale(g2);
        /* Draw a border, if selected */
        drawBorder(g2);
        /* Draw the bars */
        drawBar(g2, maxHistory, maxColor);
        drawBar(g2, actval, fillColor);
        drawBar(g2, minHistory, minColor);
        /* Draw the scale */
        drawScale(g2);
    }
    
    /** Informes the owner, that the variable value has been changed
     * @param var Variable to display
     */
    public void VarChanged(JVVar var) {
        if (var != null) {
            if (getCommVarCfg(3).getSymbol() != null &&
            getCommVarCfg(3).getSymbol().equals(var.getVarCfg().getSymbol())) {
                maxHistory = ((JVVarDouble)var).getVarValue();
            } else if (getCommVarCfg(4).getSymbol() != null &&
            getCommVarCfg(4).getSymbol().equals(var.getVarCfg().getSymbol())) {
                minHistory = ((JVVarDouble)var).getVarValue();
            }
        }
        super.VarChanged(var);
    }
    
    /** Get the content of the minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the minimum value.
     */
    public String getMinHistoryVar() {
        return getCommVarSymbol(4);
    }
    
    /** Setter for property minVar.
     * @param minVar New value of property maxVar.
     */
    public void setMinHistoryVar(String minVar) {
        setCommVarSymbol(4, minVar);
    }
    
    /** Get the content of the minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the minimum value.
     */
    public String getMaxHistoryVar() {
        return getCommVarSymbol(3);
    }
    
    /** Setter for property maxVar.
     * @param maxVar New value of property maxVar.
     */
    public void setMaxHistoryVar(String maxVar) {
        setCommVarSymbol(3, maxVar);
    }
    
    /** Get the color for the min bar
     * @return min bar color
     */
    public Color getMinColor() {
        return minColor;
    }
    
    /** Set the color for the min bar
     * @param color the new color
     */
    public void setMinColor(Color color) {
        minColor = color;
        repaint();
    }

    /** Get the color for the max bar
     * @return max bar color
     */
    public Color getMaxColor() {
        return maxColor;
    }
    
    /** Set the color for the max bar
     * @param color the new color
     */
    public void setMaxColor(Color color) {
        maxColor = color;
        repaint();
    }
    
    
    /** Main method for standalone testing */
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        JPanel p = new JPanel();
        p.setBackground(Color.gray);
        JVBar lvl = new JVBar();
        lvl.setCommVar("t1");
        lvl.setServer("localhost");
        lvl.setCommtype("org.jvisu.comm.JVSimRandomComm");
        lvl.setMaxVar("1");
        lvl.setMinVar("0");
        lvl.majortics = 5;
        lvl.minortics = 10;
        lvl.setFracDigits(2);
        lvl.minalarm = 20;
        lvl.maxalarm = 80;
        lvl.orientation = 0;
        lvl.setDesignsize(new Dimension(500,500));
        lvl.invers = false;
        lvl.setForeground(Color.BLUE);
        lvl.setLogarithmic(false);
        lvl.setFont(new Font("Default", 0, 11));
        lvl.setOpaque(true);
        lvl.drawBorder = true;
        p.add(lvl);
        frame.getContentPane().add(p);
        frame.pack();
        frame.setVisible(true);
    }    
    
}
