package org.jvisu.base;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.panel.*;
import org.jvisu.comm.*;

/**
 *  <p>package:    JVisu</p>
 *  <p>file:       JVCommStatus.java</p>
 * $Id: JVCommStatus.java 318 2007-08-21 11:23:19Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>g
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * Show the communication status.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVCommStatus extends javax.swing.JPanel implements 
                            JVBaseMembers, PropertyChangeListener, MouseListener {

    /** Position of this component */
    protected Point                 designpos;    
    /** The designmodel for this component */
    private String                  designmodel;
    /** The icon for status connected */
    private ImageIcon               imgFalse;
    /** The icon for status disconnected */
    private ImageIcon               imgTrue;
    /** The communication status */
    private boolean                 status;
    /** A dialog to display */
    private JDialog                 dialog;
    /** A list to display the status string*/
    private JList                   list;
    /** Image to draw */
    private Image                   img;
    /** Size of this component */
    protected Dimension             designsize;
    /** Zoom support */
    protected double                zoomfactor = 1.0;
    /** The fontsize designed for this component */
    protected int                   fontsize;
    
    /** Creates a new instance of JVLabel */
    public JVCommStatus() {
        designpos = new Point(0,0);
        designmodel = getClass().getName();
        imgFalse = new ImageIcon(getClass().getResource("/org/jvisu/base/icons/connectno.png"));
        imgTrue = new ImageIcon(getClass().getResource("/org/jvisu/base/icons/connected.png"));
        designsize = new Dimension (32,32);
        setPreferredSize(designsize);
        setOpaque(false);
        dialog = new JDialog();
        dialog.setModal(true);
        dialog.setTitle("Communication status");
        JScrollPane pane = new JScrollPane();
        list = new JList(new DefaultListModel());
        pane.setViewportView(list);
        dialog.getContentPane().add(pane);
        addMouseListener(this);
        addPropertyChangeListener(this);
        this.setEnabled(false);
    }

    /** Paint the components image
     *  @param g    the Graphics context to draw
     */
    public void paintComponent(Graphics g) {
        if (img == null || img.getWidth(null) != getWidth() || img.getHeight(null) != getHeight()) {
            if (status)
                img = imgTrue.getImage().getScaledInstance(getWidth(),getHeight(),java.awt.Image.SCALE_FAST);
            else
                img = imgFalse.getImage().getScaledInstance(getWidth(),getHeight(),java.awt.Image.SCALE_FAST);
        }
        g.drawImage(img, 0, 0, this);
    }
    
    /** Called by the parent component, when the component is
     *  added.
     *  Startup component
     */
    public void addNotify() {
        /* Let the superclass do its magic */
        super.addNotify();
        firePropertyChange("designpos", null, designpos);
        String[] txt = JVComm.getStatusMessages();
        DefaultListModel model = (DefaultListModel)list.getModel();
        for (int i = 0; i < txt.length; i++)
            model.add(i, txt[i]);
        JVComm.addPropertyChangeListener(this);
        status = JVComm.isStatusOK();
        //System.out.println (status);
        repaint();
    }

    /** Called by the parent component, when the component is
     *  removed.
     */
    public void removeNotify() {
        JVComm.removePropertyChangeListener(this);
    }
    
    /** Getter for property designpos.
     * @return Value of property designpos.
     */
    public Point getDesignpos() {
        return designpos;
    }
    
    /** Setter for property designpos.
     * @param newpos new position
     */
    public void setDesignpos(Point newpos) {
        Point oldpos = new Point(designpos);
        designpos.setLocation(newpos);
        firePropertyChange("designpos", oldpos, newpos);
    }
    
    /** get the designmodel for this component 
     * @return designmodel
     */
    public String getDesignModel() {
        return designmodel;
    }
    
    /** set the designmodel for this component 
     * @param model designmodel
     */
    public void setDesignModel(String model) {
        designmodel = model;
    }

    /** Get the ability of this component to have subcomponents
     * @return true, if subcomponents are allowed
     */
    public boolean isContainer() {
        return false;
    }
    
    /** Property change handler
     *  @param evt the property change event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getSource() == this) {
            String key = evt.getPropertyName();
            if (key.equals("designsize") || key.equals("zoomfactor")) {
                recalcSize();
            }
        } else { 
            if (evt.getPropertyName().equals("commstatus")) {
                status = ((Boolean) evt.getNewValue()).booleanValue();
                img = null;
                repaint();
            }
            if (evt.getPropertyName().equals("statusmsg")) {
                DefaultListModel model = (DefaultListModel)list.getModel();
                model.add(0, evt.getNewValue());
                if (model.size() > JVComm.MAX_STATUS_MSG)
                    model.remove(JVComm.MAX_STATUS_MSG);
                list.revalidate();
                if (dialog.isShowing())
                    dialog.repaint();
            }
        }
    }
    
    public void mouseClicked(MouseEvent e) {
        if (!this.isEnabled())
            return;
        dialog.pack();
        dialog.setLocationRelativeTo(this);
        dialog.setVisible(true);
    }
    
    public void mouseEntered(MouseEvent e) {
    }
    
    public void mouseExited(MouseEvent e) {
    }
    
    public void mousePressed(MouseEvent e) {
    }
    
    public void mouseReleased(MouseEvent e) {
    }
    
    /** Return the design size of this component
     * @return design size
     */
    public Dimension getDesignsize() {
        return designsize;
    }
    
    /** Set the design size of this component.
     * @param newpos new designsize
     */
    public void setDesignsize(Dimension newsize) {
        Dimension oldsize = new Dimension (designsize);
        designsize.setSize(newsize);
        firePropertyChange("designsize", oldsize, newsize);
    }

    /** Get the zoom factor of this component
     * @return zoomfactor for this component
     */
    public double getZoomfactor() {
        return zoomfactor;
    }

    /** Set the zoom factor of this component
     * @param zoom new zoomfactor for this component
     */
    public void setZoomfactor(double zoom) {
        double oldzoom = zoomfactor;
        zoomfactor = zoom;
        firePropertyChange("zoomfactor", oldzoom, zoom);
    }
    
    /** Recalc the size depending on zoom factor and designsize 
     */
    private void recalcSize() {
        Dimension d = new Dimension(designsize);
        d.setSize(zoomfactor * d.getWidth(), zoomfactor * d.getHeight());
        this.setPreferredSize(d);
        Font font = super.getFont();
        super.setFont(new Font(font.getName(), font.getStyle(), 
            (int)(zoomfactor * (double) fontsize)));
    }
    
}
