/*
 * VisuConfig.java
 *
 * Created on 21. Mai 2003, 17:28
 */

package org.jvisu.base;

import java.net.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.util.*;
import java.util.zip.*;
import java.lang.*;
import java.lang.Object.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import org.jconfig.visual.*;
import java.text.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.jconfig.visual.*;
import org.jconfig.items.*;
import org.jvisu.comm.*;
import org.jvisu.comm.var.*;
import cz.dhl.ftp.*;

/**
 *  <p>package:    JVisu</p>
 *  <p>file:       JVConfig.java</p>
 * $Id: JVConfig.java 318 2007-08-21 11:23:19Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>g
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * A panel for editing xml configs
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVConfig extends JVBean {
    
    private ConfigPanel         ConfPanel;    
    private JTree               ConfTree;
    private JScrollPane         ConfView, TreeView;
    private JSplitPane          splitter;
    private DefaultTreeModel    ConfModel; 
    DefaultMutableTreeNode      topNode;
    TreeItem                    cfgItem, topItem;
    
    /** Holds value of property configFile. */
    private String              configFile;
    /** Use local filesystem for storage instead of a JVisu Server connectoin */
    private boolean             usefs;
    /** User level for access control */
    private int                 userlevel = 0;
    /** true, if valid configuration data are available */
    private boolean             configValid;
    /** Holds value of property paramFile. */
    private String              paramFile;
    /** True, if UNIX style parameter files are used */
    private boolean             unixstyle;
    /** username for ftp access */
    private String username;
    /** password for ftp access */
    private String password;
    
    /** Creates a new instance of VisuConfig */
    public JVConfig() {
        setMinimumSize(new Dimension(200,100));
        setLayout (new BoxLayout(this, BoxLayout.X_AXIS));
        InitComponents();
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
    }
  
    /** Subclasses have to implement a stop function
     */
    public void Shutdown() {
    }

    /** Subclasses have to implement a start function
     */
    public void StartUp() {
    }
    
    /** VarChanged must be overridem !
     */
    public void VarChanged (JVVar var) {
    }
    
    /** Build node tree 
     *  This function is called recursive
     *  @param parent  parent node to analyse
     */
    private void BuildNodes (DefaultMutableTreeNode parent) {
        DefaultMutableTreeNode child = null;
        TreeItem item = (TreeItem) parent.getUserObject();
        for (int i = 0; i < item.items.size(); i++) {
            TreeItem subitem = (TreeItem) item.items.get(i);
            if (!subitem.HasName("nodeitem") && !subitem.HasName("masteritem"))
                continue;
            child = new DefaultMutableTreeNode (subitem);
            parent.add (child);
            BuildNodes (child);
        }
        if (child != null) {
            TreePath path = new TreePath (child.getPath());
            //ConfTree.makeVisible(path);    
        } 
    }
    
    /** Setup the node tree 
     */
    private void SetupTree () {
        TreeItem top = null;
        // Find the top node
        for (int i = 0; i < cfgItem.items.size(); i++) {
            top = (TreeItem) cfgItem.items.get(i);
            if (top.GetName().compareTo("topitem") == 0)
                break;
            top = null;
        }
        if (top == null)
            return;
        top.RemoveFromParent();
        cfgItem = top;
        unixstyle = top.CheckAttrValue("Fileformat","Unix");
        topNode.removeAllChildren();
        topNode.setUserObject(cfgItem);
        ConfModel.reload(topNode);
        BuildNodes(topNode);
    }
    
    /** Read a xml file into a tree
     *  @param file File to read from
     *  @return new topitem on success else null
     **/
    private TreeItem ReadFile (String file) {
        TreeItem item;
        Ftp ftp = null;
        try {
            InputStream in;
            URL ctrl = new URL (file);
            if (ctrl.getProtocol().compareTo("ftp") == 0) {
                ftp = new Ftp();
		if (!ftp.connect (ctrl.getHost(), 21)) {
			ftp.abort();
			return null;
		}
		if (!ftp.login (username, password)) {
			ftp.abort();
			return null;
		}
		String txt = ctrl.getPath();
		if (txt.charAt(0) == '/')
			txt = txt.substring(1);
		FtpFile ffile = new FtpFile (txt, ftp);
		if (ffile == null) {
			ftp.abort();
			return null;
		}
		in = ffile.getInputStream();	
            } else {
                URLConnection ctrlConnection = ctrl.openConnection ();                               
                in = ctrlConnection.getInputStream (); 
            }
            if (file.endsWith(".gz"))
                in = new GZIPInputStream(in);

            // parse the given tree items XML
            item = new TreeItem("Read", null);
            // Expand arrays !
            TreeItemHandler Handler = new TreeItemHandler(item, true);
            try { 
                SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
                p.parse(in, Handler); 
                in.close ();
            } 
            catch (Exception e) {e.printStackTrace(); item = null;}
        } catch (Exception e2) { System.out.println (e2); item = null;}
        if (ftp != null) {
		if (ftp.isConnected())
			ftp.abort();
	}
	return item;
    }

    /** Read a UNIX style parameter file
     *	@param file File to read from
     *  @param top top of item tree to store parameter values
     */
    private void ReadParameterFile (String file, TreeItem top) {
        String line;
	ItemAttr sym = new ItemAttr("symbol", null);
	ItemAttr[] attr = new ItemAttr[] {sym};
        try {
            BufferedReader in;
            URL ctrl = new URL (file);
            URLConnection ctrlConnection = ctrl.openConnection ();
            in = new BufferedReader (new InputStreamReader (ctrlConnection.getInputStream ())); 
            while ((line = in.readLine()) != null) {
            	if (line.length() == 0)
                    continue;
                if (line.charAt(0) == '#')
                    continue;
                String[] tags = line.split("=");
                if (tags == null || tags.length == 0)
                    continue;
                sym.setValue(tags[0]);
                TreeItem it = top.GetItemFollow("paramitem", attr);
            	if (it == null)
                    continue;
                if (tags.length == 1) {
                    String def = it.GetAttrValue ("default");
                    it.ModifyAttr("value", def == null ? "" : def);
                } else {
                    tags = tags[1].split("\"");
                    String val = tags[0];
                    for (int i = 1; i < tags.length; i++)
                        val += tags[i];
                    if (val.length() > 0)
                        it.ModifyAttr("value", val);
                    else
                        it.ModifyAttr("value", null);
                }
            }
            in.close();
        } catch (Exception e) {return;}
    }
   	

    /** Save the configuration tree */	
    public void SaveTree () {
        if (configValid) {
            if (!unixstyle)
                SaveTree(cfgItem, configFile);
            else
                SaveParameterFile (cfgItem, paramFile);
        }
    }
    
    /** Save the tree recursive
     * 	@param top Treenode to retrieve data
     *  @param file Filename to save
     */
    private void SaveTree (TreeItem top, String file) {
        ItemToXML encoder = top.printFollow(null, false, false);
        if (encoder == null)
            return;
        try {
            WriteFile (file, encoder.GetString());
            /* Service mode */
            if (this.userlevel == 3) {
                String dfile = file.substring(0,file.lastIndexOf('.'));
                dfile += ".default.gz";
                WriteFile (dfile, encoder.GetString());
            }
        } catch (Exception e) { }
    }
  
    /** Write file 
     *  @param file filename
     *  @param stream String which contains values
     */
    public void WriteFile (String file, String stream) {
        Ftp ftp = null;
        System.out.println(file);
        try {
            OutputStream out = null;
            URL ctrl = new URL (file);
            if (usefs) {
                out = new FileOutputStream(ctrl.getPath());
            } else {
                if (ctrl.getProtocol().compareTo("http") == 0) {
                    /*
		    URL cgi = new URL("http://"+ctrl.getHost()+"/cgi-bin/jvisufile.cgi?"+crtl.getPath()+"?"+stream);
                    URLConnection cgiConnection = cgi.openConnection();           //request to webserver
                    BufferedReader in = new BufferedReader( new InputStreamReader(cgiConnection.getInputStream())); 
                    String iL;
                    while ((iL = in.readLine()) != null){                    //read answer
                        System.out.println(iL);
                    }
                    in.close();
		    */
                } else if (ctrl.getProtocol().compareTo("ftp") == 0) {
	                ftp = new Ftp();
			if (!ftp.connect (ctrl.getHost(), 21)) {
				ftp.abort();
				return;
			}
			if (!ftp.login (username, password)) {
				ftp.abort();
				return;
			}
			String txt = ctrl.getPath();
			if (txt.charAt(0) == '/')
				txt = txt.substring(1);
			FtpFile ffile = new FtpFile (txt, ftp);
			if (ffile == null) {
				ftp.abort();
				return;
			}
			out = ffile.getOutputStream();	
                } else {
                    URLConnection ctrlConnection = ctrl.openConnection ();                               
                    out = ctrlConnection.getOutputStream (); 
                } 
            }
            
            if (out != null) {
                if (!file.endsWith(".gz")) {
                    out.write(stream.getBytes());
                    out.write("\n".getBytes());
                } else {
                    GZIPOutputStream outgz = new GZIPOutputStream(out);
                    outgz.write(stream.getBytes());
                    outgz.write("\n".getBytes());
                    outgz.finish();
                }
                out.close();
            }
            
        } catch (Exception e) {}
        if (ftp != null) {
		if (ftp.isConnected())
			ftp.abort();
	}
    }
    
    /** Save a unix style parameterfile
     *  @param top Treendoe to retrieve data
     *  @param file Filename to save
     */
    private void SaveParameterFile (TreeItem top, String file) {
        String stream = "# " + file + "\n";
        stream = SaveItemValues (top,  stream);
        WriteFile (file, stream);
    }

    /** Save Item values. Called recursive through the tree
     *  @param item Item to analyse
     *  @param stream String to write to
     */
    private String SaveItemValues (TreeItem item, String stream) {
        String symbol = item.GetAttrValue("symbol");
        String value = item.GetAttrValue("value");
        if (symbol != null) {
            symbol += "=";
            if (value != null) {
                if (value.indexOf(' ') == -1 && value.indexOf('=') == -1) 
                    symbol += value;
                else 
                    symbol += "\"" + value + "\"";
            }
            stream += symbol;
            stream +="\n";
        }
        for (int i = 0; i < item.items.size(); i++)
            stream = SaveItemValues((TreeItem)item.GetItem(i), stream);

        return stream;
    }
    
    /** Select a node
     */
    private void NodeSelect (TreeSelectionEvent e) {
        NodeSelect (false);
    }
    
    /** Select a node
     *  @param restore Restore Configpanel contents
     */
    private void NodeSelect (boolean restore) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) ConfTree.getLastSelectedPathComponent();
        if (node == null) 
            return;
        
        TreeItem it = (TreeItem)node.getUserObject();
        if (it != null)
            ConfPanel.SetConfigItems(it, restore, userlevel);
    }
    
    /** Initialize visual components */
    private void InitComponents () {
	cfgItem = new TreeItem ();
        topNode = new DefaultMutableTreeNode (cfgItem);
        ConfModel = new DefaultTreeModel(topNode);
        ConfTree = new JTree(ConfModel);
        ConfTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        // Listen for selection change
        ConfTree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                NodeSelect (e);
            } 
        });    

        TreeView = new JScrollPane(ConfTree);
        TreeView.setMinimumSize(new Dimension(100,100));

        ConfPanel = new ConfigPanel(true, BoxLayout.Y_AXIS);
        ConfView = new JScrollPane(ConfPanel,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        ConfView.setMinimumSize(new Dimension(100,100));

        add(TreeView);
        add(ConfView);
        splitter = new JSplitPane (JSplitPane.HORIZONTAL_SPLIT, TreeView, ConfView);
        splitter.setDividerLocation(200);
        add(splitter);
        
    }

    /** TODO: Implement this and JVisu server support */
    public void ChangeCommInstance (JVComm old, JVComm news) {
    }
    
    /** Getter for property configFile.
     * @return Value of property configFile.
     */
    public String getConfigFile() {
        return this.configFile;
    }
    
    /** Setter for property configFile.
     * @param configFile New value of property configFile.
     *
     */
    public void setConfigFile(String configfile) {
        configFile = configfile;
        TreeItem it = ReadFile (configFile);
        if (it == null)
            return;
        cfgItem = it;
        SetupTree();
        configValid = true;
        if (unixstyle)
            ReadParameterFile (paramFile, cfgItem);
        ConfTree.setSelectionPath(new TreePath (topNode.getPath()));
        revalidate();
    }

    /** Load backup file
     * @param configFile 
     *
     */
    public void LoadBackupFile() {
        String dfile = configFile.substring(0,configFile.lastIndexOf('.'));
        dfile += ".default.gz";
        TreeItem it = ReadFile (dfile);
        if (it == null)
            return;
        cfgItem = it;
        SetupTree();
        configValid = true;
        if (unixstyle)
            ReadParameterFile (paramFile, cfgItem);
        ConfTree.setSelectionPath(new TreePath (topNode.getPath()));
        revalidate();
    }
    
    /** Getter for property usefs.
     * @return Value of property usefs.
     */
    public boolean isUsefs() {
        return this.usefs;
    }
    
    /** Setter for property usefs.
     * @param usefs New value of property usefs.
     */
    public void setUsefs(boolean usefs) {
        this.usefs = usefs;
    }
    
    /** Getter for property userlevel.
     * @return Value of property userlevel.
     *
     */
    public int getUserlevel() {
        return this.userlevel;
    }
    
    /** Setter for property userlevel.
     * @param userlevel New value of property userlevel.
     *
     */
    public void setUserlevel(int userlevel) {
        this.userlevel = userlevel;
        NodeSelect (false);
    }
    
    /** Getter for property splitposition.
     * @return Value of property splitposition.
     *
     */
    public int getSplitposition() {
        return splitter.getDividerLocation();
    }
    
    /** Setter for property splitposition.
     * @param splitposition New value of property splitposition.
     *
     */
    public void setSplitposition(int splitposition) {
        splitter.setDividerLocation(splitposition);
        NodeSelect (false);
    }
    
    public String readParamValue (String name, ItemAttr attr[]) {
        return cfgItem.GetItemFollow(name, attr).GetAttrValue("value");
    }
    
    /** Getter for property paramFile.
     * @return Value of property paramFile.
     *
     */
    public String getParamFile() {
        return this.paramFile;
    }
    
    /** Setter for property paramFile.
     *  Read parameterfile, if a valid configuration file is loaded
     *  and the configuration parameters have unix style
     *
     * @param paramFile New value of property paramFile.
     */
    public void setParamFile(String paramFile) {
        this.paramFile = paramFile;
        if (unixstyle && configValid)
            ReadParameterFile (paramFile, cfgItem);
	NodeSelect(false);
    }
    
    /** Return the top item of the tree */
    public TreeItem getTop () {
            return cfgItem;
    }
    
    /** Getter for property username.
     * @return Value of property username.
     *
     */
    public String getUsername() {
        return this.username;
    }
    
    /** Setter for property username.
     * @param username New value of property username.
     *
     */
    public void setUsername(String username) {
        this.username = username;
    }
    
    /** Getter for property password.
     * @return Value of property password.
     *
     */
    public String getPassword() {
        return this.password;
    }
    
    /** Setter for property password.
     * @param password New value of property password.
     *
     */
    public void setPassword(String password) {
        this.password = password;
    }
    
}
