package org.jvisu.base;

import java.awt.*;
import java.text.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.comm.*;
import org.jvisu.util.param.JVParams;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *  The purpose of JVDigits is to display a rectangular shape, which contains
 *  the numerical representation of a variable. The representation can be 
 *  customized by various methods.
 *  The components has a builtin min/max compare function, which can be custumized by 
 *  absolute values or variable values, which are retrieved from the communication server
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public abstract class JVHexDez extends JVBean implements JVVarListener {
 
    /** Color for text display */    
    private Color       textColor = Color.black;

    /** Hex / dec switch */
    protected boolean   disphex;
    
    /** Create a new instance ofJVHexByte
     */
    public JVHexDez() {
        this.setDesignsize(new Dimension (100,30));
    }

    /** Subclasses have to implement a start function
     */
    public void StartUp() {
    }

    /** Subclasses have to implement a stop function
     */
    public void Shutdown() {
    }

    /** Get the symbol of the communication variable
     * @return the variable symbol of this component
     */
    public String getCommVar() {
        return getCommVarSymbol(0);
    }

    /** Set the symbol of the communication variable
     * @param symbol the variable symbol of this component
     */
    public void setCommVar(String symbol) {
        setCommVarSymbol(0, symbol);
    }

    /** Get text color
     *  @return text color
     */
    public Color getTextColor (){
        return textColor;
    }

    /** Set text color
     *  @param col text color
     */
    public void setTextColor (Color col){
        textColor = col;
    }

    /** Abstract implementation for subclasses, as they 
     * know how to construct the correct variable type
     */
    public abstract String getVarString ();

    /** VisuDigits specific paint function.
     *  Check, if a min/max check is given<br>
     *  If yes, check, if value is inside the tresholds.<br>
     *  If inside use backcolor, else use alarmcolor
     * @param g Graphics context
     */
    public void paintComponent (Graphics g) {
        
        Rectangle rect = new Rectangle(getSize());
        
        Color col = this.isOpaque() ? this.getBackground() : null;
        if (col != null) {
            g.setColor (col);
            g.fillRect (rect.x, rect.y, rect.width, rect.height);
        }
  
        String txt = getVarString();
        g.setColor (textColor);
        g.setFont(this.getFont());

        // retrieve textsize
        int fheight = g.getFontMetrics ().getHeight ();
        int fwidth = g.getFontMetrics ().stringWidth (txt);
        
        int x = 0; 
        // set x startpoint depending on alignement
        switch (getAligncontent()) {
            // align left
            case JVAlignment.JVISU_ALIGNLEFT: x = g.getFontMetrics ().charWidth(' '); break;
            // align center
            case JVAlignment.JVISU_ALIGNCENTER: x = (rect.width - fwidth) / 2; break;
            // align right
            default: x = rect.width - fwidth - g.getFontMetrics ().charWidth(' ');
        }
        g.drawString (txt, x, (rect.height + fheight / 2) / 2);
    }
    
    /** Getter for property disphex.
     * @return Value of property disphex.
     *
     */
    public boolean isDisphex() {
        return this.disphex;
    }
    
    /** Setter for property disphex.
     * @param disphex New value of property disphex.
     *
     */
    public void setDisphex(boolean disphex) {
        this.disphex = disphex;
        repaint ();
    }
}
