package org.jvisu.base;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.net.*;
import java.text.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.panel.*;
import org.jvisu.comm.*;
import org.jvisu.util.param.JVParams;
import org.jvisu.comm.var.*;

/** Visual component to plot a numerical (double) array
 *
 * $Id: JVDouble.java 275 2004-04-19 08:20:55Z tgl $
 *
 *  <p>Copyright 2005 Erwin Rol <ero@penguntronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *
 * @author Erwin Rol <ero@pengutronix.de>
 * @version 1.0
 */
public class JVPlot extends JVBean implements JVVarListener 
{
    public static final int BORDER_WIDTH = 2;

    public static final int  COMM_VAR_MINX = 0;
    public static final int  COMM_VAR_MAXX = 1;
    public static final int  COMM_VAR_MINY = 2;
    public static final int  COMM_VAR_MAXY = 3;    
    public static final int  COMM_VAR_VAL  = 4;
 
    private double              plotdata[];
    
    protected double            minxval = 0;
    protected double            maxxval = 1000;
    protected double            minyval = 0;
    protected double            maxyval = 100;

    private boolean             minxvalByVariable;
    private boolean             maxxvalByVariable;
    private boolean             minyvalByVariable;
    private boolean             maxyvalByVariable;
   

    /** Constant to calc log10 */
    public static final double  LOG10_FAC = 1 / Math.log(10);
    /** Number of major tics */
    protected int               majorticsx = 5;
    /** Number of minor tics */
    protected int               minorticsx = 2;
    /** Logarithmic scaling */
    protected boolean           isXLog = false;
    /** Number of major tics */
    protected int               majorticsy = 2;
    /** Number of minor tics */
    protected int               minorticsy = 5;
    /** Logarithmic scaling */
    protected boolean           isYLog = false;

    /** Length of the major ticks scale lines */
    protected int               scalelen = 4;

    /** The Rectangle of the bar */
    protected Rectangle         plotRect = new Rectangle();
    
    protected Color		plotBackgroundColor = Color.white;
    protected Color		plotLineColor = Color.red;
    //protected Color		plotForegroundColor = Color.black;
    protected Color		plotGridColor = Color.gray;

    protected NumberFormat      numFormat;

    /** Recalculate  */
    protected boolean           recalc = true;

    /** Create a new instance of VisuDigits
     */
    public JVPlot() {
        // Build commvariables for act, xmin, xmax, ymn, ymax
        //addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLEARRAY, 0, COMM_VAR_VAL));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, COMM_VAR_MINX));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, COMM_VAR_MAXX));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, COMM_VAR_MINY));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, COMM_VAR_MAXY));

        // Instantiate a new Numberformat
        numFormat = NumberFormat.getInstance(java.util.Locale.US);
        numFormat.setMinimumIntegerDigits(1);
        numFormat.setMaximumIntegerDigits(10);
        numFormat.setMinimumFractionDigits(0);
        numFormat.setMaximumFractionDigits(0);

	plotdata = new double[1];
	plotdata[0] = 0.0;

	this.setDesignsize(new Dimension(200,200));
    }

    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        if (evt.getSource() == this) {
            /* Recalc and repaint */
            if (key.equals("foreground") ||
                key.equals("font") ||
                key.equals("preferredSize") ||
                key.equals("minxval") ||
                key.equals("maxxval") ||
                key.equals("minyval") ||
                key.equals("maxyval") ||
		key.equals("plotBackgroundColor") ||
		key.equals("plotLineColor") ||
                key.equals("plotGridColor") ||
                key.equals("logarithmicx") ||
                key.equals("majorticsx") ||
                key.equals("minorticsx") ||
                key.equals("logarithmicy") ||
                key.equals("majorticsy") ||
                key.equals("minorticsy") ) {
                    recalc = true;
                    repaint();
            }
        }
    }
    

   /** Paint this component
     *  @param g the graphics context
     */
    public void paintComponent (Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);

	/* Fill the background, if opaque is true */
        Dimension size = getSize();
        if (isOpaque()) {
            g.setColor(getBackground());
            g.fillRect(0, 0, size.width, size.height);
        }

	drawBackground(g);
	
	drawPlot(g);

    }

    protected void drawBackground (Graphics g) {
        g.setFont(getFont());
        /* retrieve textsize */
        scalelen = g.getFontMetrics().getHeight();

        /* Find the value with the maximum text width for Y axis labels*/
        int wmax = 0;
        for (int i = 0; i <= majorticsy; i++) {
            int w = g.getFontMetrics().stringWidth(calcMajorTextY(i));
            if (w > wmax)
                wmax = w;
        }

	int lastw = g.getFontMetrics().stringWidth(calcMajorTextX(majorticsx));

	Dimension size = getSize();
	plotRect.x = wmax + scalelen;
	plotRect.y = scalelen / 2;
	plotRect.width = size.width - plotRect.x - (lastw / 2);
	plotRect.height = size.height - plotRect.y - (scalelen  * 2);

	g.setColor(plotBackgroundColor);
	g.fillRect(plotRect.x, plotRect.y, plotRect.width, plotRect.height);

        /* X Axis */
        int x = plotRect.x;
        int y = plotRect.y + plotRect.height;
        int len = plotRect.width;
        //g.drawLine(x, y, x + len, y);
        int ticlen = scalelen - 2;
        for (int i = 0; i <= majorticsx; i++) {
            int pos = x + (int) (calcMajorTickX(i) * (double) len);
	    g.setColor(getForeground());
            g.drawLine(pos, y, pos, y + ticlen);
	    g.setColor(plotGridColor);
            g.drawLine(pos, y, pos, y - plotRect.height);
            if (i != majorticsx) {
                for (int j = 1; j < minorticsx; j++) {
                    int mpos = x + (int) (calcMinorTickX(i, j) * (double) len);
		    g.setColor(getForeground());
                    g.drawLine(mpos, y, mpos, y + ticlen / 2);
		    g.setColor(plotGridColor);
		    g.drawLine(mpos, y, mpos, y - plotRect.height);
                }
            }
            String txt = calcMajorTextX(i);
            int w = g.getFontMetrics().stringWidth(txt);
            int fonty = y + 2 * scalelen - g.getFontMetrics().getDescent();
	    g.setColor(getForeground());
            g.drawString(txt, pos - w/2, fonty);
        }

	/* Y Axis */
	x = plotRect.x;
        y = plotRect.y + plotRect.height;
        len = -plotRect.height;
        //g.drawLine(x, y, x, y + len);
        ticlen = scalelen - 2;
        for (int i = 0; i <= majorticsy; i++) {
            int pos = y + (int) (calcMajorTickY(i) * (double) len);
	    g.setColor(getForeground());
            g.drawLine(x - ticlen, pos, x, pos);
            g.setColor(plotGridColor);
            g.drawLine(x, pos, x + plotRect.width, pos);
            if (i != majorticsy) {
                for (int j = 1; j < minorticsy; j++) {
                    int mpos = y + (int) (calcMinorTickY(i, j) * (double) len);
                    g.setColor(getForeground());
                    g.drawLine(x - ticlen / 2, mpos, x, mpos);
                    g.setColor(plotGridColor);
		    g.drawLine(x, mpos, x + plotRect.width, mpos);
                }
            }
            String txt = calcMajorTextY(i);
            int w = g.getFontMetrics().stringWidth(txt);
            int fontx = x - scalelen - w;
            int fonty = pos + (scalelen - g.getFontMetrics().getDescent()) / 2;
            g.setColor(getForeground());
            g.drawString(txt, fontx, fonty);
        }

	g.setColor(getForeground());
	g.drawRect(plotRect.x, plotRect.y, plotRect.width, plotRect.height);
    }

    protected void drawPlot(Graphics g) 
    {
	if (plotdata.length < 1)
		return;

	int count = plotRect.width;

        int plx[] = new int[count];
	int ply[] = new int[count];

	for(int i = 0; i < count; i++ ) {
		plx[i] = plotRect.x + i;

		int x = (int)(((double)plotdata.length / (double)count) * (double)i);
		if (x < 0)
			x = 0;

		if (x >= plotdata.length)
			x = plotdata.length - 1;		

		ply[i] = (int)((plotdata[x] - minyval) * ((double)plotRect.height / ((double)maxyval - (double)minyval)));
		ply[i] = (plotRect.y + plotRect.height) - ply[i];

		if (ply[i] < plotRect.y)
			ply[i] = plotRect.y;

		if (ply[i] > (plotRect.y + plotRect.height))
			ply[i] = plotRect.y + plotRect.height;
	}

	g.setColor(plotLineColor);
        g.drawPolyline(plx, ply, count);
    }

    /** Draw the order, if selected
     *  @param g2 the graphics context
     */
    protected void drawBorder (Graphics2D g2) {
//        if (drawBorder) {
 //           border.paintBorder(this, g2, plotRect.x - BORDER_WIDTH, plotRect.y - 2,
  //              plotRect.width + 4, plotRect.height + 4);
    //    }
    }

    /** Calculate the tick position for a major tick
     *  @param major major tick number
     *  @param invers if true, the position is negated
     *  @return the tick position as fraction of the full scale
     */
    protected double calcMajorTickX(int major) {
        double f = (double) major / (double) majorticsx;
        return f;
    }

    /** Calculate the tick text for a major tick
     *  @param int major tick number
     *  @return the description text for this tick
     */
    protected String calcMajorTextX(int tic) {
        double f = (double) tic / (double) majorticsx;
        if (isXLog) {
            if (minxval <= 0 || maxxval < minxval)
                return "";
            double max = LOG10_FAC * Math.log (maxxval);
            double min = LOG10_FAC * Math.log (minxval);
            f = Math.pow(10, (max - min) * f + min);
        } else {
            f = f * (maxxval - minxval) + minxval;
        }
        return numFormat.format(f);
    }

    /** Calculate the tick position for a minor tick
     *  @param major major tick number
     *  @param minor minor tick number
     *  @param invers if true, the position is negated
     *  @return the tick position as fraction of the full scale
     */
    protected double calcMinorTickX(int major, int minor) {
        double f = (double) major / (double) majorticsx;
        if (isXLog) {
            if (minxval <= 0 || maxxval < minxval)
                return 0;
            // calc distance to next major tick
            double dist = LOG10_FAC / (double) majorticsx;
            f += dist * Math.log(10 * ((double) minor)/ (double) minorticsx);
        } else {
            f += (double) minor / (double) (majorticsx * minorticsx);
        }
        return f;
    }


    /** Calculate the tick position for a major tick
     *  @param major major tick number
     *  @param invers if true, the position is negated
     *  @return the tick position as fraction of the full scale
     */
    protected double calcMajorTickY(int major) {
        double f = (double) major / (double) majorticsy;
        return f;
    }

    /** Calculate the tick text for a major tick
     *  @param int major tick number
     *  @return the description text for this tick
     */
    protected String calcMajorTextY(int tic) {
        double f = (double) tic / (double) majorticsy;
        if (isYLog) {
            if (minyval <= 0 || maxyval < minyval)
                return "";
            double max = LOG10_FAC * Math.log (maxyval);
            double min = LOG10_FAC * Math.log (minyval);
            f = Math.pow(10, (max - min) * f + min);
        } else {
            f = f * (maxyval - minyval) + minyval;
        }
        return numFormat.format(f);
    }

    /** Calculate the tick position for a minor tick
     *  @param major major tick number
     *  @param minor minor tick number
     *  @param invers if true, the position is negated
     *  @return the tick position as fraction of the full scale
     */
    protected double calcMinorTickY(int major, int minor) {
        double f = (double) major / (double) majorticsy;
        if (isXLog) {
            if (minyval <= 0 || maxyval < minyval)
                return 0;
            // calc distance to next major tick
            double dist = LOG10_FAC / (double) majorticsy;
            f += dist * Math.log(10 * ((double) minor)/ (double) minorticsy);
        } else {
            f += (double) minor / (double) (majorticsy * minorticsy);
        }
        return f;
    }

    /** Subclasses have to implement a start function
     */
    public void StartUp() {
    }
    
    /** Subclasses have to implement a stop function
     */
    public void Shutdown() {
    }
    

    /** Set variable value, send it to server, if possible
     */
    public void setValue(double val) {
        setCommVarValue(COMM_VAR_VAL, new Double( val ));
    }
    
    /** Get the symbol of the communication variable
     * @return the variable symbol of this component
     */
    public String getCommVar() {
        return getCommVarSymbol(COMM_VAR_VAL);
    }
    
    /** Set the symbol of the communication variable
     * @param symbol the variable symbol of this component
     */
    public void setCommVar(String symbol) {
        setCommVarSymbol(COMM_VAR_VAL, symbol);
        VarChanged(null);
    }

    
    /** Return setting, if xminval is controlled by a variable
     *  @return true, if xminval is controlled by a variable
    */
    public boolean getMinxvalByVariable () {
        return minxvalByVariable;
    }

    /** Control, if xminval is controlled by a variable
     *  @param val true, if xminval is controlled by a variable
    */
    public void setMinxvalByVariable (boolean val) {
        minxvalByVariable = val;
        setCommVarEnable(COMM_VAR_MINX, val);
        VarChanged(null);
    }

    /** Return setting, if xmaxval is controlled by a variable
     *  @return true, if xmaxval is controlled by a variable
    */
    public boolean getMaxxvalByVariable () {
        return maxxvalByVariable;
    }

    /** Control, if xminval is controlled by a variable
     *  @param val true, if xminval is controlled by a variable
    */
    public void setMaxxvalByVariable (boolean val) {
        maxxvalByVariable = val;
        setCommVarEnable(COMM_VAR_MAXX, val);
        VarChanged(null);
    }
    
    /** Return setting, if yminval is controlled by a variable
     *  @return true, if yminval is controlled by a variable
    */
    public boolean getMinyvalByVariable () {
        return minyvalByVariable;
    }

    /** Control, if yminval is controlled by a variable
     *  @param val true, if yminval is controlled by a variable
    */
    public void setMinyvalByVariable (boolean val) {
        minyvalByVariable = val;
        setCommVarEnable(COMM_VAR_MINY, val);
        VarChanged(null);
    }

    /** Return setting, if ymaxval is controlled by a variable
     *  @return true, if ymaxval is controlled by a variable
    */
    public boolean getMaxyvalByVariable () {
        return maxyvalByVariable;
    }

    /** Control, if yminval is controlled by a variable
     *  @param val true, if yminval is controlled by a variable
    */
    public void setMaxyvalByVariable (boolean val) {
        maxyvalByVariable = val;
        setCommVarEnable(COMM_VAR_MAXY, val);
        VarChanged(null);
    }

    /** Get the content of the x minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the x minimum value.
     */
    public String getMinxVar() {
        if (!minxvalByVariable)
            return Double.toString(minxval);
        else
            return getCommVarSymbol(COMM_VAR_MINX);
    }
    
    /** Setter for property minVar.
     * @param minVar New value of property maxVar.
     */
    public void setMinxVar(String minVar) {
        if (minxvalByVariable)
            setCommVarSymbol(COMM_VAR_MINX, minVar);
        else
            setMinxVal(JVParams.StringToDouble(minVar, minxval));
    }
    
    /** Get the content of the minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the minimum value.
     */
    public String getMaxxVar() {
        if (!maxxvalByVariable)
            return Double.toString(maxxval);
        else
            return getCommVarSymbol(COMM_VAR_MAXX);
    }
    
    /** Setter for property maxVar.
     * @param maxVar New value of property maxVar.
     */
    public void setMaxxVar(String maxVar) {
        if (maxxvalByVariable)
            setCommVarSymbol(COMM_VAR_MAXX, maxVar);
        else
            setMaxxVal(JVParams.StringToDouble(maxVar, maxxval));
    }

    /** Get the content of the y minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the y minimum value.
     */
    public String getMinyVar() {
        if (!minyvalByVariable)
            return Double.toString(minyval);
        else
            return getCommVarSymbol(COMM_VAR_MINY);
    }
    
    /** Setter for property yminVar.
     * @param minVar New value of property yminVar.
     */
    public void setMinyVar(String minVar) {
        if (minyvalByVariable)
            setCommVarSymbol(COMM_VAR_MINY, minVar);
        else
            setMinyVal(JVParams.StringToDouble(minVar, minyval));
    }
    
    /** Get the content of the Y maximum value.
     *  This can be a numerical value or a variable symbol
     *  @return the Y maximum value.
     */
    public String getMaxyVar() {
        if (!maxyvalByVariable)
            return Double.toString(maxyval);
        else
            return getCommVarSymbol(COMM_VAR_MAXY);
    }
    
    /** Setter for property ymaxVar.
     * @param maxVar New value of property maxVar.
     */
    public void setMaxyVar(String maxVar) {
        if (maxyvalByVariable)
            setCommVarSymbol(COMM_VAR_MAXY, maxVar);
        else
            setMaxyVal(JVParams.StringToDouble(maxVar, maxyval));
    }

    public int getArraySize() {
	return plotdata.length;
    }

    public void setArraySize(int size) {
	//JVVarCfg cfg = getCommVarCfg(COMM_VAR_VAL);
	plotdata = new double[ size ]; 
     	/*
	setCommVar( COMM_VAR_VAL, new JVVarCfg(cfg.getSymbol(), cfg.getRate(), cfg.getType(), size, COMM_VAR_VAL) );

	setCommVarSymbol(COMM_VAR_VAL, cfg.getSymbol());

	*/
	setCommVarSize(COMM_VAR_VAL, size);

        VarChanged(null);
    }


    /** Get the fill color for the arc
     * @return the fill color of the arc
     */
    public Color getPlotBackgroundColor() {
        return plotBackgroundColor;
    }

    /** Setter for property fillColor.
     * @param emptyColor New value of property fillColor.
     */
    public void setPlotBackgroundColor(Color color) {
        plotBackgroundColor = color;
        repaint();
    }


    /** Get the fill color for the arc
     * @return the fill color of the arc
     */
    public Color getPlotLineColor() {
        return plotLineColor;
    }

    /** Setter for property fillColor.
     * @param emptyColor New value of property fillColor.
     */
    public void setPlotLineColor(Color color) {
        plotLineColor = color;
        repaint();
    }


    /** Get the fill color for the arc
     * @return the fill color of the arc
     */
    public Color getPlotGridColor() {
        return plotGridColor;
    }

    /** Setter for property fillColor.
     * @param emptyColor New value of property fillColor.
     */
    public void setPlotGridColor(Color color) {
        plotGridColor = color;
        repaint();
    }


    /** Get the number of minor tics on the scale
     * @return number of minor tics
     */
    public int getMinorticsx() {
        return minorticsx;
    }

    /** Set the number of minor tics on the scale
     * @param tics the number of minor tics
     */
    public void setMinorticsx(int tics) {
        int old = minorticsx;
        minorticsx = tics;
        firePropertyChange ("minorticsx", old, tics);
    }

    /** Get the number of major tics on the scale
     * @return number of major tics
     */
    public int getMajorticsx() {
        return majorticsx;
    }

    /** Set the number of major tics on the scale
     * @param tics the number of major tics
     */
    public void setMajorticsx(int tics) {
        int old = majorticsx;
        majorticsx = tics;
        firePropertyChange ("majorticsx", old, tics);
    }

    /** Get the parameter for logarithmic scaling
     * @return true, if logarithmic scaling is active
     */
    public boolean isLogarithmicx() {
        return isXLog;
    }

    /** Set the parameter for logarithmic scaling
     * @param log true, if logarithmic scaling is desired
     */
    public void setLogarithmicx(boolean log) {
        boolean old = isXLog;
        isXLog = log;
        firePropertyChange ("logarithmicx", old, isXLog);
    }



    /** Get the number of minor tics on the scale
     * @return number of minor tics
     */
    public int getMinorticsy() {
        return minorticsy;
    }

    /** Set the number of minor tics on the scale
     * @param tics the number of minor tics
     */
    public void setMinorticsy(int tics) {
        int old = minorticsy;
        minorticsy = tics;
        firePropertyChange ("minorticsy", old, tics);
    }

    /** Get the number of major tics on the scale
     * @return number of major tics
     */
    public int getMajorticsy() {
        return majorticsy;
    }

    /** Set the number of major tics on the scale
     * @param tics the number of major tics
     */
    public void setMajorticsy(int tics) {
        int old = majorticsy;
        majorticsy = tics;
        firePropertyChange ("majorticsy", old, tics);
    }

    /** Get the parameter for logarithmic scaling
     * @return true, if logarithmic scaling is active
     */
    public boolean isLogarithmicy() {
        return isYLog;
    }

    /** Set the parameter for logarithmic scaling
     * @param log true, if logarithmic scaling is desired
     */
    public void setLogarithmicy(boolean log) {
        boolean old = isYLog;
        isYLog = log;
        firePropertyChange ("logarithmicy", old, isYLog);
    }


    /** set the X minimum value 
     *  @param min the new minimum value
     */
    private void setMinxVal(double min) {
         double old = minxval;
         minxval = min;
         firePropertyChange("minxval", old, minxval);
    }

    /** set the X maximum value 
     *  @param max the new maximum value
     */
    private void setMaxxVal(double max) {
         double old = maxxval;
         maxxval = max;
         firePropertyChange("maxxval", old, maxxval);
    }

    /** set the Y minimum value 
     *  @param min the new minimum value
     */
    private void setMinyVal(double min) {
         double old = minyval;
         minyval = min;
         firePropertyChange("minyval", old, minyval);
    }

    /** set the Y maximum value 
     *  @param max the new maximum value
     */
    private void setMaxyVal(double max) {
         double old = maxyval;
         maxyval = max;
         firePropertyChange("maxyval", old, maxyval);
    }

    private void setCommVarVal(double val)
    {
	plotdata = new double[1];
        plotdata[0] = val;
    }

    private void setCommVarVal(double val[])
    {
	plotdata = new double[ val.length ];
	for (int i = 0; i < val.length; i++)
	        plotdata[i] = val[i];
    }

    private void setCommVarVal(float val[])
    {
	plotdata = new double[ val.length ];
	for (int i = 0; i < val.length; i++)
	        plotdata[i] = (double)val[i];
    }

    private void setCommVarVal(byte val[])
    {
	plotdata = new double[ val.length ];
	for (int i = 0; i < val.length; i++)
	        plotdata[i] = (double)val[i];
    }

    private void setCommVarVal(short val[])
    {
	plotdata = new double[ val.length ];
	for (int i = 0; i < val.length; i++)
	        plotdata[i] = (double)val[i];
    }

    private void setCommVarVal(int val[])
    {
	plotdata = new double[ val.length ];
	for (int i = 0; i < val.length; i++)
	        plotdata[i] = (double)val[i];
    }

    private void CommVarValChanged(JVVar var) 
    {
	switch(var.getVarCfg().getType()) {
		case JVVar.T_DOUBLE:
			setCommVarVal( (double) ((JVVarDouble)var).getVarValue() );
			break;
		case JVVar.T_BYTE:
			setCommVarVal( (double) ((JVVarByte)var).getVarValue() );
			break;
		case JVVar.T_SHORT:
			setCommVarVal( (double) ((JVVarShort)var).getVarValue() );
			break;
		case JVVar.T_INT:
			setCommVarVal( (double) ((JVVarInt)var).getVarValue() );
			break;
		case JVVar.T_BYTEARRAY:
			setCommVarVal( ((JVVarByteArray)var).getVarValue() );
			break;
		case JVVar.T_SHORTARRAY:
			setCommVarVal( ((JVVarShortArray)var).getVarValue() );
			break;
		case JVVar.T_INTARRAY:
			setCommVarVal( ((JVVarIntArray)var).getVarValue() );
			break;
		case JVVar.T_FLOATARRAY:
			setCommVarVal( ((JVVarFloatArray)var).getVarValue() );
			break;
		case JVVar.T_DOUBLEARRAY:
			setCommVarVal( ((JVVarDoubleArray)var).getVarValue() );
			break;
		default:
			setCommVarVal( 0.0 );
			break;
	}
/*		
	System.out.println(var.getVarCfg().getSymbol() );

	if (plotdata.length != var.getVarCfg().getSize()) {
		JVVarCfg cfg = var.getVarCfg();
		setCommVar( COMM_VAR_VAL, new JVVarCfg(cfg.getSymbol(), cfg.getRate(), cfg.getType(), plotdata.length, COMM_VAR_VAL) );
		VarChanged(null);
	}
*/
    }    

    /** Informes the owner, that the variable value has been changed
     * @param var Variable to display
     */
    public void VarChanged(JVVar var) {
        if (var != null) {
            if (getCommVarCfg(COMM_VAR_VAL).getSymbol() != null &&
                getCommVarCfg(COMM_VAR_VAL).getSymbol().equals(var.getVarCfg().getSymbol())) 
            {
		CommVarValChanged(var);
            } 
            else if (getCommVarCfg(COMM_VAR_MINX).getSymbol() != null &&
                     getCommVarCfg(COMM_VAR_MINX).getSymbol().equals(var.getVarCfg().getSymbol())) 
            {
                setMinxVal(((JVVarDouble)var).getVarValue());
            } 
            else if (getCommVarCfg(COMM_VAR_MAXX).getSymbol() != null &&
                     getCommVarCfg(COMM_VAR_MAXX).getSymbol().equals(var.getVarCfg().getSymbol())) 
            {
                setMaxxVal(((JVVarDouble)var).getVarValue());
            } 
            else if (getCommVarCfg(COMM_VAR_MINY).getSymbol() != null &&
                     getCommVarCfg(COMM_VAR_MINY).getSymbol().equals(var.getVarCfg().getSymbol())) 
            {
                setMinyVal(((JVVarDouble)var).getVarValue());
            } 
            else if (getCommVarCfg(COMM_VAR_MAXY).getSymbol() != null &&
                     getCommVarCfg(COMM_VAR_MAXY).getSymbol().equals(var.getVarCfg().getSymbol())) 
            {
                setMaxyVal(((JVVarDouble)var).getVarValue());
            }
        }
        repaint();
    }


    /** Main method for standalone testing */
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        JVPlot lvl = new JVPlot();
        lvl.setCommVar("t1");
        lvl.setServer("localhost");
        lvl.setCommtype("org.jvisu.comm.JVSimRandomComm");

        lvl.setMaxxVar("1");
        lvl.setMinxVar("0.1");
        lvl.setMaxyVar("1");
        lvl.setMinyVar("0.1");

        lvl.setFont(new Font("Default", 0, 11));
        lvl.setPreferredSize(new Dimension(300,300));
        //lvl.drawBorder = true;
        JPanel p = new JPanel();
        p.setBackground(Color.gray);
        p.add(lvl);
        frame.getContentPane().add(p);
        frame.pack();
        frame.setVisible(true);
    }
}
