package org.jvisu.comm;

import java.util.*;
import org.jvisu.comm.var.*;

/** JVisu simulated communication
 *
 *  <p>$Id: JVSimComm.java 168 2003-11-18 08:56:07Z tgl $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  <p>This class provides the simulation based communication functions for 
 *  JVisu. This functuionality can be used for prototyping and promotion.
 *  </p>
 *  <p><b>TODO:</b> Check all variable related stuff for thread safety
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
 * @since JVisu 1.0
*/
public class JVSimComm extends JVComm {
  
    /** communication status */    
    protected boolean active;
    
    /** Creates a new instance of JVCommSim.
     */
    public JVSimComm() {
        JVComm.setStatusMessage(this, 0, "Created");
    }
    
    /** Check, if the instance is matching 
     *  @param cfg configuration to compare
     *  @return true, if matching
     */
    protected boolean isMatching (JVCommCfg cfg) {
        return (active && 
            cfg.getType() != null &&
            cfg.getType().compareTo(this.getClass().getName()) == 0 &&
            cfg.getPort() == this.commcfg.getPort() );
    }
    
    /** Check, if the instance is reuseable
     *  @param cfg configuration to compare
     *  @return true, if reuseable
     */
    protected boolean isReusable (JVCommCfg cfg) {
        return (!active &&
                cfg.getType() != null &&
                cfg.getType().compareTo(this.getClass().getName()) == 0);
    }

    /** Register a JVisu variable with this instance
     * @param var JVisu variable, which has to be registered
     */    
    protected void RegisterVar(JVVar var) {
    }
    
    /** Unregister a JVisu variable with this instance
     * @param var JVisu variable, which has to be unregistered
     */    
    protected void UnRegisterVar(JVVar var) {
    }

    /** Send a variable value to the server.
     * This is a dummy function, as we are simulating the communication
     * @param var JVisu variable which has to be sent to the server
     * @return true on success
     */    
    protected boolean SendVar(JVVar var) {
        return true;
    }
    
    /** Start communication */
    protected void startComm() {
        active = true;
        JVComm.setStatusMessage(this, 0, "Started");
    }
     
    /** Stop communication to server.
     * This can only be done, if all variables are unregistered.
     * For simulated communication it means just to stop the
     * simulation thread.
     */    
    protected boolean stopComm() { 
        active = false;
        setConnected (false);
        JVComm.setStatusMessage(this, 0, "Stopped");
        return true;
    }
    
    /** Get the activation status
     * @return true, if active
     */    
    public boolean isActive() {
        return active;
    }
    
    /** Check, if the given configuration is valid for
     * this communication type
     * @param cfg the configuration to check
     * @return true, if valid
     */
    protected boolean isValid(JVCommCfg cfg) {
        return true;
    }
}
