package org.jvisu.comm.var;

import java.nio.*;

/** JVisu variable representing a Stream.
 *
 * <p>package:  org.jvisu.comm.var</p>
 * <p>file:    JVVarStream.java</p>
 * <p>$Id: JVVarStream.java 275 2004-04-19 08:20:55Z tgl $ </p>
 * <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See file LICENCE in this
 * directory</p>
 *
 * <p>Change Log:</p>
 * <ul>
 * <li> 1.0 Initial inmplimentation </li>
 * </ul>
 * <p>
 * Implementation of abstract class JVVar representing a Stream variable for
 * communication with a JVisu communication server<br>
 * A stream variable is different to the other data types, as the stream buffer 
 * is not used for permanent data storage. It is used to tunnel communication
 * data. It can be used e.g. to monitor serial output data.
 * </p>
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVVarStream extends JVVar {
    
    private ByteBuffer value;
    
    /** Creates a new instance of JVVarStream
     */
    public JVVarStream() {
        value = ByteBuffer.allocate (256);
        value.clear();
    }
    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean SetVarValue(String str) {
        value.put(str.getBytes());
        value.flip();
        NotifyListeners(true);
        return true;
    }

    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
     */
    public boolean SetVarValue(ByteBuffer buf) {
        value.put(buf);
        value.flip();
        NotifyListeners(true);
        return true;
    }
    
    /** Set the variable value from an object
     *  @param obj  Object containing the variable value
     *  @return     true, if successful
    */
    public boolean SetVarValue(Object obj) {
        return false;
    }

    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public ByteBuffer GetVarValue(ByteBuffer buf) {
        buf.put(value);
        value.compact();
        return buf;
    }

    /** Get the string representation of the variable.
     *  This is a dummy implementation, as it cannot be
     *  represented by a string. 
     *  @return     Constant string "JVVarStream"
     */
    public String toString () {
        return "JVVarStream";
    }
}
